﻿using NetMQ;
using System;
using System.IO;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;

namespace NetMQAndProtocolBuffer.PublisherPattern.Server
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
        }

        private void Btn_SendMsg_Click(object sender, RoutedEventArgs e)
        {
            bool stopRequested = false; //stop signal
            const int zipToSubscribeTo = 10001; //Message prefix for server to indentify.

            WriteLine("Publishing weather updates...");

            var sendTask = Task.Factory.StartNew(() =>
            {
                using (var context = NetMQContext.Create())
                using (var publisher = context.CreatePublisherSocket()) 
                {
                    publisher.Bind("tcp://127.0.0.1:5556");
                    var rng = new Random();
                    while (!stopRequested)
                    { 
                        int zipcode = rng.Next(10000,10010); //Relpace: rng.Next(0, 99999);
                        int temperature = rng.Next(-80, 135);
                        int relhumidity = rng.Next(0, 90);

                        #region Protobuf
                        var weatherMsg = new NetMQAndProtocolBuffer.PBProtocol.WeatherMessage()
                        {
                            Command = PBProtocol.WeatherMessage.CommandType.Weather,
                            Content = string.Format("{0} {1} {2}", zipcode, temperature, relhumidity), 
                        };
                        
                        using (var sm = new MemoryStream())
                        {
                            ProtoBuf.Serializer.Serialize<NetMQAndProtocolBuffer.PBProtocol.WeatherMessage>(sm, weatherMsg);
                            publisher.Send(sm.ToArray());
                        }
                        #endregion

                        // publisher.Send(string.Format("{0} {1} {2}", zipcode, temperature, relhumidity));
                       
                        WriteLine(string.Format("Publisher send message: {0} {1} {2}", zipcode, temperature, relhumidity));
                        System.Threading.Thread.Sleep(100);
                    }
                }
            });
        }

        /// <summary>
        /// Writes the line.
        /// </summary>
        /// <param name="msg">The MSG.</param>
        private void WriteLine(string msg)
        {
            this.Dispatcher.Invoke(new Action(() => { this.Txt_Output.Text += msg + System.Environment.NewLine; }));
        }

        /// <summary>
        /// Handles the TextChanged event of the Txt_Output control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="TextChangedEventArgs"/> instance containing the event data.</param>
        private void Txt_Output_TextChanged(object sender, TextChangedEventArgs e)
        {
            this.Txt_Output.ScrollToEnd();
        }
    }
}
